﻿using Microsoft.EntityFrameworkCore;
using SewingDressesContracts.BindingModels;
using SewingDressesContracts.SearchModels;
using SewingDressesContracts.StoragesContracts;
using SewingDressesContracts.ViewModels;
using SewingDressesDatabaseImplement.Models;
using SewingDressesDataModels.Models;

namespace SewingDressesDatabaseImplement.Implements
{
	public class ShopStorage : IShopStorage
	{
		public ShopViewModel? Delete(ShopBindingModel model)
		{
			using var context = new SewingDressesDatabase();
			var element = context.Shops.FirstOrDefault(x => x.Id == model.Id);
			if (element != null)
			{
				context.Shops.Remove(element);
				context.SaveChanges();
				return element.GetViewModel;
			}
			return null;
		}

		public ShopViewModel? GetElement(ShopSearchModel model)
		{
			if (string.IsNullOrEmpty(model.Name) && !model.Id.HasValue)
			{
				return null;
			}
			using var context = new SewingDressesDatabase();
			return context.Shops.Include(x => x.Dresses).ThenInclude(x => x.Dress).FirstOrDefault(x => model.Id.HasValue && x.Id == model.Id)?.GetViewModel;
		}

		public List<ShopViewModel> GetFilteredList(ShopSearchModel model)
		{
			if (string.IsNullOrEmpty(model.Name))
			{
				return new();
			}
			using var context = new SewingDressesDatabase();
			return context.Shops
				.Include(x => x.Dresses)
				.ThenInclude(x => x.Dress)
				.Select(x => x.GetViewModel)
				.Where(x => x.ShopName.Contains(model.Name ?? string.Empty))
				.ToList();
		}

		public List<ShopViewModel> GetFullList()
		{
			using var context = new SewingDressesDatabase();
			return context.Shops
			.Include(x => x.Dresses)
				.ThenInclude(x => x.Dress)
				.Select(x => x.GetViewModel)
				.ToList();
		}

		public ShopViewModel? Insert(ShopBindingModel model)
		{
			using var context = new SewingDressesDatabase();
			try
			{
				var newShop = Shop.Create(context, model);
				if (newShop == null)
				{
					return null;
				}
				if (context.Shops.Any(x => x.ShopName == newShop.ShopName))
				{
					throw new Exception("Не должно быть два магазина с одним названием");
				}
				context.Shops.Add(newShop);
				context.SaveChanges();
				return newShop.GetViewModel;
			}
			catch
			{
				throw;
			}
		}

		public ShopViewModel? Update(ShopBindingModel model)
		{
			using var context = new SewingDressesDatabase();
			var shop = context.Shops.FirstOrDefault(x => x.Id == model.Id);
			if (shop == null)
			{
				return null;
			}
			try
			{
				if (context.Shops.Any(x => (x.ShopName == model.ShopName && x.Id != model.Id)))
				{
					throw new Exception("Не должно быть два магазина с одним названием");
				}
				shop.Update(model);
				shop.UpdateDresses(context, model);
				context.SaveChanges();
				return shop.GetViewModel;
			}
			catch
			{
				throw;
			}
		}
		public bool SellDresses(IDressModel model, int count)
		{
			if (model == null)
				return false;
			using var context = new SewingDressesDatabase();
			using var transaction = context.Database.BeginTransaction();
			List<ShopDress> lst = new List<ShopDress>();
			foreach (var el in context.ShopDresses.Where(x => x.DressId == model.Id))
			{
				int dif = count;
				if (el.Count < dif)
					dif = el.Count;
				el.Count -= dif;
				count -= dif;
				if (el.Count == 0)
				{
					lst.Add(el);
				}
				if (count == 0)
					break;
			}
			if (count > 0)
			{
				transaction.Rollback();
				return false;
			}
			foreach (var el in lst)
			{
				context.ShopDresses.Remove(el);
			}
			context.SaveChanges();
			transaction.Commit();
			return true;
		}
	}
}
