﻿using SewingDressesContracts.BindingModels;
using SewingDressesContracts.SearchModels;
using SewingDressesContracts.StoragesContracts;
using SewingDressesContracts.ViewModels;
using SewingDressesDataModels.Models;
using SewingDressesFileImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SewingDressesFileImplement.Implements
{
    public class ShopStorage : IShopStorage
    {
        private readonly DataFileSingleton _source;
        public ShopStorage()
        {
            _source = DataFileSingleton.GetInstance();
        }
        public List<ShopViewModel> GetFullList()
        {
            return _source.Shops.Select(x => x.GetViewModel).ToList();
        }
        public List<ShopViewModel> GetFilteredList(ShopSearchModel model)
        {
            if (string.IsNullOrEmpty(model.Name))
            {
                return new();
            }
            return _source.Shops.Where(x => x.ShopName.Contains(model.Name)).Select(x => x.GetViewModel).ToList();
            
        }
        public ShopViewModel? GetElement(ShopSearchModel model)
        {
            if (string.IsNullOrEmpty(model.Name) && !model.Id.HasValue)
            {
                return null;
            }
            return _source.Shops.FirstOrDefault(x => (!string.IsNullOrEmpty(model.Name) && x.ShopName == model.Name) || (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
        }
        public ShopViewModel? Insert(ShopBindingModel model)
        {
            model.Id = _source.Shops.Count > 0 ? _source.Shops.Max(x => x.Id) + 1 : 1;
            var newShop = Shop.Create(model);
            if (newShop == null)
            {
                return null;
            }
            _source.Shops.Add(newShop);
            _source.SaveShops();
            return newShop.GetViewModel;
        }
        public ShopViewModel? Update(ShopBindingModel model)
        {
            var component = _source.Shops.FirstOrDefault(x => x.Id == model.Id);
            if (component == null)
            {
                return null;
            }
            component.Update(model);
            _source.SaveShops();
            return component.GetViewModel;
        }
        public ShopViewModel? Delete(ShopBindingModel model)
        {
            var element = _source.Shops.FirstOrDefault(x => x.Id == model.Id);
            if (element != null)
            {
                _source.Shops.Remove(element);
                _source.SaveShops();
                return element.GetViewModel;
            }
            return null;
        }
        public bool CheckAvailability(int dressId, int count)
        {
            int store = _source.Shops.Select(x => x.ShopDresses.Select(y => (y.Value.Item1.Id == dressId ? y.Value.Item2 : 0)).Sum()).Sum();
            return store >= count;
        }
        public bool SellDresses(IDressModel model, int count)
        {
            var dres = _source.Dresses.FirstOrDefault(x => x.Id == model.Id);

            if (dres == null || !CheckAvailability(model.Id, count))
            {
                return false;
            }

            for (int i = 0; i < _source.Shops.Count; i++)
            {
                var shop = _source.Shops[i];
                var dresses = shop.ShopDresses;
                foreach (var dress in dresses.Where(x => x.Value.Item1.Id == dres.Id))
                {
                    var selling = Math.Min(dress.Value.Item2, count);
                    dresses[dress.Value.Item1.Id] = (dress.Value.Item1, dress.Value.Item2 - selling);

                    count -= selling;

                    if (count <= 0)
                    {
                        break;
                    }
                }
                shop.Update(new ShopBindingModel
                {
                    Id = model.Id,
                    ShopName = shop.ShopName,
                    Adress = shop.Adress,
                    MaxCount = shop.MaxCount,
                    DateOpen = shop.DateOpen,
                    ShopDresses = dresses
                });
            }
            _source.SaveShops();
            return true;
        }
    }
}
