﻿using SewingDressesContracts.BindingModels;
using SewingDressesContracts.ViewModels;
using SewingDressesDataModels.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace SewingDressesFileImplement.Models
{
    public class Shop : IShopModel
    {
        public int Id { get; private set; }
        public string ShopName { get; private set; }
        public string Adress { get; private set; }
        public DateTime DateOpen { get; private set; }
        public int MaxCount { get; private set; }
        public Dictionary<int, int> Dresses { get; private set; } = new();
        private Dictionary<int, (IDressModel, int)>? _shopDresses = null;
        public Dictionary<int, (IDressModel, int)> ShopDresses
        { 
            get
            {
                if (_shopDresses == null)
                {
                    var source = DataFileSingleton.GetInstance();
                    _shopDresses = Dresses.ToDictionary(x => x.Key, y => ((source.Dresses.FirstOrDefault(z => z.Id == y.Key) as IDressModel)!, y.Value));
                }
                return _shopDresses;
            }
        }
        public static Shop? Create(ShopBindingModel model)
        {
            if (model == null)
            {
                return null;
            }
            return new Shop()
            {
                Id = model.Id,
                ShopName = model.ShopName,
                Adress = model.Adress,
                DateOpen = model.DateOpen,
                MaxCount = model.MaxCount,
                Dresses = model.ShopDresses.ToDictionary(x => x.Key, x => x.Value.Item2)
            };
        }
        public static Shop? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }
            return new Shop()
            {
                Id = Convert.ToInt32(element.Attribute("Id")!.Value),
                ShopName = element.Element("ShopName")!.Value,
                Adress = element.Element("Adress")!.Value,
                MaxCount = Convert.ToInt32(element.Element("MaxCount")!.Value),
                DateOpen = Convert.ToDateTime(element.Element("DateOpen")!.Value),
                Dresses = element.Element("ShopDresses")!.Elements("ShopDress").ToDictionary(x => Convert.ToInt32(x.Element("Key")?.Value), x => Convert.ToInt32(x.Element("Value")?.Value))
            };

        }
        public void Update(ShopBindingModel? model)
        {
            if (model == null)
            {
                return;
            }
            ShopName = model.ShopName;
            Adress = model.Adress;
            DateOpen = model.DateOpen;
            MaxCount = model.MaxCount;
            if (model.ShopDresses.Count > 0)
            {
                Dresses = model.ShopDresses.ToDictionary(x => x.Key, x => x.Value.Item2);
                _shopDresses = null;
            }
        }
        public ShopViewModel GetViewModel => new()
        {
            Id = Id,
            ShopName = ShopName,
            Adress = Adress,
            DateOpen = DateOpen,
            MaxCount = MaxCount,
            ShopDresses = ShopDresses
        };

        public XElement GetXElement => new XElement("Shop",
            new XAttribute("Id", Id),
            new XElement("ShopName", ShopName),
            new XElement("Adress", Adress),
            new XElement("DateOpen", DateOpen),
            new XElement("MaxCount", MaxCount),
            new XElement("ShopDresses", Dresses.Select(x => new XElement("ShopDress", new XElement("Key", x.Key), new XElement("Value", x.Value))).ToArray()));
    }
}
